use ezcad::pen::Pen;
use serde::{Deserialize, Serialize};

use self::pen::{ClonePen, PatchPen, PatternPen};

pub mod pen;

#[derive(Debug, Serialize, Deserialize)]
pub enum Operation {
    PatchPen(PatchPen),
    ClonePen(ClonePen),
    PatternPen(PatternPen),
}

pub trait Operations {
    fn apply(&self, target: &mut Vec<Pen>);
}

impl Operations for Vec<Operation> {
    fn apply(&self, target: &mut Vec<Pen>) {
        for op in self {
            match op {
                Operation::PatchPen(x) => x.patch(target),
                Operation::ClonePen(x) => x.clone(target),
                Operation::PatternPen(x) => x.pattern(target),
            }
        }
    }
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "PascalCase")]
pub struct Config {
    pub ops: Vec<Operation>,
}
