use std::fmt::Debug;

use binrw::{BinRead, BinWrite};
use diff::Diff;

use crate::{
    field_of::FieldOf,
    types::{Field, Point, F64, U32},
};

use super::ObjectCore;

#[derive(BinRead, BinWrite, Diff, PartialEq)]
#[diff(attr(
    #[derive(Debug, PartialEq)]
))]
pub struct Circle {
    pub core: ObjectCore,
    #[brw(magic(6u32))] // Number of following fields in struct
    pub origin: FieldOf<Point>,
    pub radius: F64,
    pub start_angle: F64, // Radians
    pub clockwise: U32,
    _unknown_1: [Field; 2],
}

// Custom Debug implementation to only print known fields
impl Debug for Circle {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_struct("Circle")
            .field("core", &self.core)
            .field("origin", &self.origin)
            .field("radius", &self.radius)
            .field("start_angle", &self.start_angle)
            .field("clockwise", &self.clockwise)
            .finish()
    }
}
