use std::fmt::Debug;

use binrw::{BinRead, BinWrite};
use diff::Diff;

use crate::{
    field_of::FieldOf,
    types::{Field, Point, F64, U32},
};

use super::ObjectCore;

#[derive(BinRead, BinWrite, Diff, PartialEq)]
#[diff(attr(
    #[derive(Debug, PartialEq)]
))]
pub struct Ellipse {
    pub core: ObjectCore,
    #[brw(magic(8u32))] // Number of following fields in struct
    pub clockwise: U32,
    pub corner_a: FieldOf<Point>,
    pub corner_b: FieldOf<Point>,
    pub start_angle: F64, // Radians
    pub end_angle: F64,   // Radians
    _unknown_1: [Field; 2],
    pub open_curve: U32,
}

// Custom Debug implementation to only print known fields
impl Debug for Ellipse {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_struct("Ellipse")
            .field("core", &self.core)
            .field("clockwise", &self.clockwise)
            .field("corner_a", &self.corner_a)
            .field("corner_b", &self.corner_b)
            .field("start_angle", &self.start_angle)
            .field("end_angle", &self.end_angle)
            .field("open_curve", &self.open_curve)
            .finish()
    }
}
