use std::fmt::Debug;

use binrw::{BinRead, BinWrite};
use diff::Diff;

use crate::{
    field_of::FieldOf,
    types::{Field, Point, F64, U32},
};

use super::ObjectCore;

#[derive(BinRead, BinWrite, Diff, PartialEq)]
#[diff(attr(
    #[derive(Debug, PartialEq)]
))]
pub struct Polygon {
    pub core: ObjectCore,
    #[brw(magic(10u32))] // Number of following fields in struct
    pub invert_shape: U32,
    pub corner_a: FieldOf<Point>,
    pub corner_b: FieldOf<Point>,
    pub offset_cx: F64,
    pub offset_cy: F64,
    pub offset_dx: F64,
    pub offset_dy: F64,
    pub edges: U32,
    _unknown_1: [Field; 2],
}

// Custom Debug implementation to only print known fields
impl Debug for Polygon {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_struct("Polygon")
            .field("core", &self.core)
            .field("invert_shape", &self.invert_shape)
            .field("corner_a", &self.corner_a)
            .field("corner_b", &self.corner_b)
            .field("offset_cx", &self.offset_cx)
            .field("offset_cy", &self.offset_cy)
            .field("offset_dx", &self.offset_dx)
            .field("offset_dy", &self.offset_dy)
            .field("edges", &self.edges)
            .finish()
    }
}
