use std::fmt::Debug;

use binrw::{BinRead, BinWrite};
use diff::Diff;

use crate::{
    field_of::FieldOf,
    types::{Field, Point, F64},
};

use super::ObjectCore;

#[derive(BinRead, BinWrite, Diff, PartialEq)]
#[diff(attr(
    #[derive(Debug, PartialEq)]
))]
pub struct Rectangle {
    pub core: ObjectCore,
    #[brw(magic(8u32))] // Number of following fields in struct
    pub corner_a: FieldOf<Point>,
    pub corner_b: FieldOf<Point>,
    pub round_bottom_left: F64,
    pub round_bottom_right: F64,
    pub round_top_right: F64,
    pub round_top_left: F64,
    _unknown_1: [Field; 2],
}

// Custom Debug implementation to only print known fields
impl Debug for Rectangle {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_struct("Rectangle")
            .field("core", &self.core)
            .field("corner_a", &self.corner_a)
            .field("corner_b", &self.corner_b)
            .field("round_bottom_left", &self.round_bottom_left)
            .field("round_bottom_right", &self.round_bottom_right)
            .field("round_top_right", &self.round_top_right)
            .field("round_top_left", &self.round_top_left)
            .finish()
    }
}
